import subprocess
import os

class SoundExtractor:
    TMP_FILE = '/tmp/gazeru_tmp'
    def extract(self, video):
        file_handler = open(self.TMP_FILE, 'wb')
        file_handler.write(video)
        file_handler.close()
        return self._extract()

    def get_sound_type(self):
        return self._get_sound_type()

    def _extract(self):
        raise NotImplementedError()

    def _get_sound_type(self):
        raise NotImplementedError()

class Mp3ExtractorFromFlv(SoundExtractor):
    def _extract(self):
        SOUND_FILE = '{0}.mp3'.format(self.TMP_FILE)
        subprocess.call('ffmpeg -i {0} -acodec copy {1}'.format(self.TMP_FILE, SOUND_FILE), shell=True)
        file_handler = open(SOUND_FILE, 'rb')
        sound = file_handler.read()
        file_handler.close()
        os.remove(self.TMP_FILE)
        os.remove(SOUND_FILE)
        return sound

    def _get_sound_type(self):
        return 'mp3'

class Mp3ExtractorFromSwf(SoundExtractor):
    def _extract(self):
        SOUND_FILE = '{0}.mp3'.format(self.TMP_FILE)
        subprocess.call('swfextract -m {0} -o {1}'.format(self.TMP_FILE, sound_file), shell=True)
        file_handler = open(SOUND_FILE, 'rb')
        sound = file_handler.read()
        file_handler.close()
        os.remove(self.TMP_FILE)
        os.remove(SOUND_FILE)
        return sound

    def _get_sound_type(self):
        return 'mp3'

class M4aExtractorFromMp4(SoundExtractor):
    def _extract(self):
        SOUND_FILE = '{0}.mp4'.format(self.TMP_FILE)
        subprocess.call('ffmpeg -i {0} -vn -acodec copy {1}'.format(self.TMP_FILE, SOUND_FILE), shell=True)
        file_handler = open(SOUND_FILE, 'rb')
        sound = file_handler.read()
        file_handler.close()
        os.remove(self.TMP_FILE)
        os.remove(SOUND_FILE)
        return sound

    def _get_sound_type(self):
        return 'mp4'

class SoundExtractorFactory:
    def build(self, video, video_type):
        if (video_type == 'flv'):
            return Mp3ExtractorFromFlv()
        elif (video_type == 'mp4'):
            return M4aExtractorFromMp4()
        elif (video_type == 'swf'):
            return Mp3ExtractorFromSwf()



import os
import argparse
import json
from nicovideo_api import NicovideoAPI
from sound_extractor import SoundExtractorFactory
from config import Config
from gazeru import Gazeru

if __name__ == '__main__':
    argparser = argparse.ArgumentParser(description='gazeru is niconico downloader')
    argparser.add_argument('-l', '--list',help='display registered mylist', action='store_true')
    argparser.add_argument('-a', '--add', help='add mylist')
    argparser.add_argument('-u', '--user', nargs=2, help='set user and password')
    argparser.add_argument('-d', '--directory', help='set download directory')
    args = argparser.parse_args()

    gazeru = Gazeru()
    if args.list:
        print('{0:10} {1:10} {2}'.format('id', 'creator', 'title'))
        for mylist in gazeru.get_mylists():
            print('{0:10} {1:10} {2}'.format(mylist['id'], mylist['creator'], mylist['title']))

    elif args.add:
        mylist_id = args.add
        try:
            gazeru.add_mylist(mylist_id)
        except AlreadyRegisteredMylistError:
            print('{0} is already registed'.format(mylist_id))

    elif args.user:
        user = args.user[0]
        password = args.user[1]
        gazeru.set_account(user, password)

    elif args.directory:
        """引数で渡されたディレクトリ を config ファイルに書き込んで、
        該当のディレクトリに.gazeruファイルを設置する
        .gazeruファイルは mylist_id: [{ダウンロードされたvideo_id: ファイル名}] という形式で書き込まれる
        """
        directory = args.directory
        gazeru.set_direcctory(directory)

        dot_gazeru = '{0}/.gazeru'.format(directory)
        with open(dot_gazeru, 'w') as file:
            config = Config()
            mylists = config.get_mylists()
            file.write(json.dumps({mylist['id']:[] for mylist in mylists}))
        config = Config()
        config.set_directory(directory)

    else:
        gazeru.pull()



import requests
import urllib.parse
import re
from pyquery import PyQuery as pq

class NicovideoAPI:
    def __init__(self, mailaddress, password):
        self.login(mailaddress, password)

    def login(self, mailaddress, password):
        response = requests.post(
            "https://secure.nicovideo.jp/secure/login?site=niconico",
            data = {'mail_tel': mailaddress,
                    'password': password},
            allow_redirects = False
        )
        self._cookies = response.cookies

    def get_thumb_info(self, video_id):
        response = requests.get("http://ext.nicovideo.jp/api/getthumbinfo/{0}".format(video_id))
        dom = pq(response.content, parser='xml')
        return {'video_id': dom('video_id').text(),
                'title': dom('title').text(),
                'description': dom('desctiption').text(),
                'thumbnail_url': dom('thumbnail_url').text(),
                'first_retrieve': dom('first_retrieve').text(),
                'length': dom('length').text(),
                'movie_type': dom('movie_type').text(),
                'size_high': dom('size_high').text(),
                'size_low': dom('size_low').text(),
                'view_counter': dom('view_counter').text(),
                'comment_num': dom('comment_num').text(),
                'mylist_counter': dom('mylist_counter').text(),
                'last_res_body': dom('last_res_body').text(),
                'watch_url': dom('watch_url').text(),
                'thumb_type': dom('thumb_type').text(),
                'embeddable': dom('embeddable').text(),
                'no_live_play': dom('no_live_play').text(),
                'tags': [{'tag': tag.text(), 'lock': tag.attr('lock')} for tag in dom('tags').items('tag')],
                'user_id': dom('user_id').text(),
                'user_nickname': dom('user_nickname'),
                'user_icon_url': dom('user_icon_url')}

    def get_flv(self, video_id):
        flv_url = self._get_flv_url(video_id)
        response = requests.get(flv_url, cookies=self._cookies)
        content = response.content
        return content

    def get_mylist_info(self, mylist_id):
        mylist_xml = requests.get('http://www.nicovideo.jp/mylist/{0}?rss=2.0&lkang=ja-jp'.format(mylist_id))
        dom = pq(mylist_xml.content, parser='xml')
        title = dom('channel > title').text()
        creator = dom('channel > dc\:creator').text()
        mylist = {'id': mylist_id,
                  'title': title,
                  'creator': creator,
                  'video_list': [re.search(r'[^/]*$', link.text).group(0) for link in dom('channel item link')]}
        return mylist

    def _get_flv_url(self, video_id):
        self._cookies.update(self._get_nicohistory_cookie(video_id))
        getflv_url = 'http://flapi.nicovideo.jp/api/getflv/{0}'.format(video_id)
        if video_id[:2] == 'nm':
            getflv_url = 'http://flapi.nicovideo.jp/api/getflv/{0}?as3=1'.format(video_id)
        response = requests.get(getflv_url, cookies=self._cookies)
        response_body = urllib.parse.unquote(response.text)
        flv_url = re.search(r'url=([^&]+)', response_body).group(1)
        return flv_url

    def _get_nicohistory_cookie(self, video_id):
        response = requests.get('http://www.nicovideo.jp/watch/' + video_id, cookies=self._cookies)
        return response.cookies



import os
import json
import logging
from logging import FileHandler, Formatter
from .config import Config
from .nicovideo_api import NicovideoAPI
from .sound_extractor import SoundExtractorFactory
from .exception import *

class Gazeru:
    def __init__(self):
        self.config = Config()
        self.niconico = NicovideoAPI(self.config.get_user(), self.config.get_password())
        self.sound_extractor_factory = SoundExtractorFactory()
        self.dot_gazeru = '{0}/.gazeru'.format(self.config.get_directory())
        self.GAZEL_HOME = '{0}/.gazeru'.format(os.environ['HOME'])
        self.setup_logger()

    def setup_logger(self):
        logging.basicConfig(level=logging.DEBUG, format='%(asctime)s- %(name)s - %(levelname)s - %(message)s')
        self.logger = logging.getLogger(__name__)
        file_handler = FileHandler('{0}/log/gazeru.log'.format(self.GAZEL_HOME), 'a+')
        file_handler.level = logging.INFO
        file_handler.formatter = Formatter(fmt='%(asctime)s %(message)s', datefmt='%Y/%m/%d %p %I:%M:%S')
        self.logger.addHandler(file_handler)
        return self

    def get_mylists(self):
        return self.config.get_mylists()

    def set_account(self, user, password):
        self.config.set_user(user)
        self.config.set_password(password)
        self.niconico.login(self.config.get_user(), self.config.get_password())
        self.logger.info('set user and password')
        return self

    def set_directory(self, directory):
        self.config.set_directory(directory)
        self.dot_gazeru = '{0}/.gazeru'.format(self.config.get_directory())
        self.logger.info('set directory to {0}'.format(directory))
        return self

    def add_mylist(self, mylist_id):
        registerd_mylist_id_list = [mylist['id'] for mylist in self.config.get_mylists()]
        if mylist_id in registerd_mylist_id_list:
            raise AlreadyRegisteredMylistError()
        mylist_info = self.niconico.get_mylist_info(mylist_id)
        self.config.add_mylist({'id': mylist_info['id'], 'creator': mylist_info['creator'], 'title': mylist_info['title']})
        self.logger.info('add mylist {0}'.format(mylist_id))

    def pull(self):
        if not os.path.exists(self.dot_gazeru):
            self.create_dot_gazeru()
            self.logger.info('create {0}'.format(self.dot_gazeru))
        mylists = self.config.get_mylists()
        all_mylist_info = {mylist['id']: self.niconico.get_mylist_info(mylist['id']) for mylist in mylists}
        all = {mylist_info['id']: mylist_info['video_list'] for mylist_id,mylist_info in all_mylist_info.items()}
        with open(self.dot_gazeru, 'r') as file:
            done = json.loads(file.read())

        not_yet_downloads = {mylist_id:
                             [video_id for video_id in video_id_list
                              if mylist_id not in done.keys() or video_id not in [d['video_id'] for d in done[mylist_id]]]
                             for mylist_id,video_id_list in all.items()}
        self.logger.info('not_yet_downloads is {0}'.format(not_yet_downloads))
        for mylist_id,video_id_list in not_yet_downloads.items():
            for video_id in video_id_list:
                try:
                    video_info, video = self.download_video(video_id)
                    sound,sound_type = self.extract_sound(video, video_info)
                    sound_file_path = '{0}/{1}'.format(self.config.get_directory(), all_mylist_info[mylist_id]['creator'])
                    os.makedirs(sound_file_path, exist_ok=True)
                    sound_file = '{0}.{1}'.format(self.escape_slash(video_info['title']), sound_type)
                    full_sound_file = '{0}/{1}'.format(sound_file_path, sound_file)
                    self.logger.info('start writing {0} to {1}'.format(video_id, full_sound_file))
                    with open(full_sound_file, 'wb') as file:
                        file.write(sound)
                    self.logger.info('finished writing {0} to {1}'.format(video_id, full_sound_file))
                    self.logger.info('start updating .gazeru')
                    if mylist_id not in done.keys():
                        done[mylist_id] = []
                    done[mylist_id].append({'video_id': video_id, 'title': sound_file})
                    with open(self.dot_gazeru, 'w') as file:
                        file.write(json.dumps(done))
                    self.logger.info('finished updating .gazeru')
                except Exception as e:
                    self.logger.critical(e.value)
                    self.logger.error('{0} is not downloaded'.format(video_id))

    def escape_slash(self, title):
        return title.replace("/", "／")

    def extract_sound(self, video, video_info):
        self.logger.info('start exstacting {0}'.format(video_info['video_id']))
        sound_extractor = self.sound_extractor_factory.build(video, video_info['movie_type'])
        result = (sound_extractor.extract(video), sound_extractor.get_sound_type())
        self.logger.info('finished exstracting {0}'.format(video_info['video_id']))
        return result

    def download_video(self, video_id):
        self.logger.info('start downloading {0}'.format(video_id))
        result = (self.niconico.get_thumb_info(video_id), self.niconico.get_flv(video_id))
        self.logger.info('finished downloading {0}'.format(video_id))
        return result

    def create_dot_gazeru(self):
        with open(self.dot_gazeru, 'w') as file:
            file.write(json.dumps({mylist['id']:[] for mylist in self.config.get_mylists()}))
        return self



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='gazeru',
    version = '1.0.0.a1',
    description='A crawler for niconico',
    long_description=long_description,
    url = 'http://github.com/roronya/gazeru',
    author='roronya',
    author_email='roronya628@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    keywords='niconico crawler',
    packages=find_packages(exclude=['tests*']),
    install_requires=['requests', 'pyquery'],
    entry_points = {
        'console_scripts': [
            "gazeru=gazeru.main"
        ]
    }
)
